{% load formatting %} {% if changed_data %}
<div class="changed-data">
  <p><b>Confirm Values:</b></p>
  <table>
    <tr>
      <th>Field</th>
      <th>{% if add %} Default {% else %} Current {% endif %}Value</th>
      <th>New Value</th>
    </tr>
    {% for field, values in changed_data.items %}
    <tr>
      <td>{% verbose_name obj field %}</td>
      <td>{{ values.0|format_change_data_field_value }}</td>
      <td>{{ values.1|format_change_data_field_value }}</td>
    </tr>
    {% endfor %}
  </table>
</div>
{% endif %}
